@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2016  Olivier Churlaud <olivier@churlaud.com>
 # Copyright 2014  Alex Merry <alex.merry@kdemail.net>
 # Copyright 2014  Aurélien Gâteau <agateau@kde.org>
 # Copyright 2014  Alex Turbov <i.zaufi@gmail.com>
+# Copyright 2016  Olivier Churlaud <olivier@churlaud.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -37,6 +37,7 @@ import shutil
 import subprocess
 import tempfile
 import sys
+import xml.etree.ElementTree as ET
 
 import jinja2
 
@@ -45,6 +46,9 @@ if sys.version_info.major < 3:
 else:
     from urllib.parse import urljoin
 
+import xml.etree.ElementTree as xmlET
+import json
+
 from kapidox import utils
 try:
     from kapidox import depdiagram
@@ -54,6 +58,11 @@ except ImportError:
 
 from .doxyfilewriter import DoxyfileWriter
 
+
+## @package kapidox.generator
+#
+# The generator
+
 __all__ = (
     "Context",
     "generate_apidocs",
@@ -88,6 +97,7 @@ class Context(object):
         'srcdir',
         'tagfiles',
         'dependency_diagram',
+        'copyright',
         # Output
         'outputdir',
         'htmldir',
@@ -129,13 +139,12 @@ def create_jinja_environment(doxdatadir):
 def process_toplevel_html_file(outputfile, doxdatadir, products, title,
                                api_searchbox=False):
 
-    products.sort(key=lambda x: x['name'].lower())
+    products.sort(key=lambda x: x.fancyname.lower())
     mapping = {
             'resources': './resources',
             'api_searchbox': api_searchbox,
             # steal the doxygen css from one of the frameworks
             # this means that all the doxygen-provided images etc. will be found
-            'doxygencss': products[0]['outputdir'] + '/html/doxygen.css',
             'title': title,
             'breadcrumbs': {
                 'entries': [
@@ -151,6 +160,11 @@ def process_toplevel_html_file(outputfile, doxdatadir, products, title,
     with codecs.open(outputfile, 'w', 'utf-8') as outf:
         outf.write(tmpl.render(mapping))
 
+    tmpl2 = create_jinja_environment(doxdatadir).get_template('search.html')
+    search_output = "search.html"
+    with codecs.open(search_output, 'w', 'utf-8') as outf:
+        outf.write(tmpl2.render(mapping))
+
 
 def process_subgroup_html_files(outputfile, doxdatadir, groups, available_platforms, title,
                                 api_searchbox=False):
@@ -159,9 +173,6 @@ def process_subgroup_html_files(outputfile, doxdatadir, groups, available_platfo
         mapping = {
             'resources': '../resources',
             'api_searchbox': api_searchbox,
-            # steal the doxygen css from one of the frameworks
-            # this means that all the doxygen-provided images etc. will be found
-            'doxygencss': group['libraries'][0]['outputdir'] + '/html/doxygen.css',
             'title': title,
             'breadcrumbs': {
                 'entries': [
@@ -171,7 +182,7 @@ def process_subgroup_html_files(outputfile, doxdatadir, groups, available_platfo
                     },
                     {
                         'href': './index.html',
-                        'text': group['fancyname']
+                        'text': group.fancyname
                     }
                     ]
                 },
@@ -179,17 +190,22 @@ def process_subgroup_html_files(outputfile, doxdatadir, groups, available_platfo
             'available_platforms': sorted(available_platforms),
         }
 
-        if not os.path.isdir(group['name']):
-            os.mkdir(group['name'])
-        outputfile = group['name']+'/index.html'
+        if not os.path.isdir(group.name):
+            os.mkdir(group.name)
+        outputfile = group.name + '/index.html'
         tmpl = create_jinja_environment(doxdatadir).get_template('subgroup.html')
         with codecs.open(outputfile, 'w', 'utf-8') as outf:
             outf.write(tmpl.render(mapping))
 
+        tmpl2 = create_jinja_environment(doxdatadir).get_template('search.html')
+        search_output = group.name + "/search.html"
+        with codecs.open(search_output, 'w', 'utf-8') as outf:
+            outf.write(tmpl2.render(mapping))
+
 
 def create_dirs(ctx):
     ctx.htmldir = os.path.join(ctx.outputdir, HTML_SUBDIR)
-    ctx.tagfile = os.path.join(ctx.htmldir, ctx.modulename + '.tags')
+    ctx.tagfile = os.path.join(ctx.htmldir, ctx.fwinfo.fancyname + '.tags')
 
     if not os.path.exists(ctx.outputdir):
         os.makedirs(ctx.outputdir)
@@ -215,21 +231,24 @@ def load_template(path):
 
 
 def find_tagfiles(docdir, doclink=None, flattenlinks=False, exclude=None, _depth=0):
-    """Find Doxygen-generated tag files in a directory
+    """Find Doxygen-generated tag files in a directory.
 
     The tag files must have the extention .tags, and must be in the listed
     directory, a subdirectory or a subdirectory named html of a subdirectory.
 
-    docdir       -- the directory to search
-    doclink      -- the path or URL to use when creating the documentation
-                    links; if None, this will default to docdir
-    flattenlinks -- if this is True, generated links will assume all the html
-                    files are directly under doclink; if False (the default),
-                    the html files are assumed to be at the same relative
-                    location to doclink as the tag file is to docdir; ignored
-                    if doclink is not set
+    Args:
+        docdir:       (string) the directory to search.
+        doclink:      (string) the path or URL to use when creating the
+                      documentation links; if None, this will default to
+                      docdir. (optional, default None)
+        flattenlinks: (bool) if True, generated links will assume all the html
+                      files are directly under doclink; else the html files are
+                      assumed to be at the same relative location to doclink as
+                      the tag file is to docdir; ignored if doclink is not set.
+                      (optional, default False)
 
-    Returns a list of pairs of (tag_file,link_path)
+    Returns:
+        A list of pairs of (tag_file,link_path).
     """
 
     if not os.path.isdir(docdir):
@@ -283,18 +302,20 @@ def search_for_tagfiles(suggestion=None, doclink=None, flattenlinks=False, searc
 
     At least one of docdir or searchpaths must be given for it to find anything.
 
-    suggestion   -- the first place to look (will complain if there are no
-                    documentation tag files there)
-    doclink      -- the path or URL to use when creating the documentation
-                    links; if None, this will default to docdir
-    flattenlinks -- if this is True, generated links will assume all the html
-                    files are directly under doclink; if False (the default),
-                    the html files are assumed to be at the same relative
-                    location to doclink as the tag file is to docdir; ignored
-                    if doclink is not set
-    searchpaths  -- other places to look for documentation tag files
+    Args:
+        suggestion:   the first place to look (will complain if there are no
+                      documentation tag files there)
+        doclink:      the path or URL to use when creating the documentation
+                      links; if None, this will default to docdir
+        flattenlinks: if this is True, generated links will assume all the html
+                      files are directly under doclink; if False (the default),
+                      the html files are assumed to be at the same relative
+                      location to doclink as the tag file is to docdir; ignored
+                      if doclink is not set
+        searchpaths:  other places to look for documentation tag files
 
-    Returns a list of pairs of (tag_file,link_path)
+    Returns:
+        A list of pairs of (tag_file,link_path)
     """
 
     if not suggestion is None:
@@ -323,9 +344,12 @@ def menu_items(htmldir, modulename):
     Looks for a set of standard Doxygen files (like namespaces.html) and
     provides menu text for those it finds in htmldir.
 
-    htmldir -- the directory the HTML files are contained in
+    Args:
+        htmldir:    (string) the directory the HTML files are contained in.
+        modulname:  (string) the name of the library
 
-    Returns a list of maps with 'text' and 'href' keys
+    Returns:
+        A list of maps with 'text' and 'href' keys.
     """
     entries = [
             {'text': 'Main Page', 'href': 'index.html'},
@@ -355,20 +379,21 @@ def parse_dox_html(stream):
     The HTML files produced by Doxygen with our custom header and footer files
     look like this:
 
-        <!--
-        key1: value1
-        key2: value2
-        ...
-        -->
-        <html>
-        <head>
-        ...
-        </head>
-        <body>
-        ...
-        </body>
-        </html>
-
+    @code
+    <!--
+    key1: value1
+    key2: value2
+    ...
+    -->
+    <html>
+    <head>
+    ...
+    </head>
+    <body>
+    ...
+    </body>
+    </html>
+    @endcode
 
     The parser fills the dict from the top key/value block, and add the content
     of the body to the dict using the "content" key.
@@ -417,8 +442,10 @@ def postprocess_internal(htmldir, tmpl, mapping):
 
     Performs text substitutions on each line in each .html file in a directory.
 
-    htmldir -- the directory containing the .html files
-    mapping -- a dict of mappings
+    Args:
+        htmldir: (string) the directory containing the .html files.
+        mapping: (dict) a dict of mappings.
+
     """
     for name in os.listdir(htmldir):
         if name.endswith('.html'):
@@ -447,9 +474,11 @@ def postprocess_internal(htmldir, tmpl, mapping):
 def build_classmap(tagfile):
     """Parses a tagfile to get a map from classes to files
 
-    tagfile -- the Doxygen-generated tagfile to parse
+    Args:
+        tagfile: the Doxygen-generated tagfile to parse.
 
-    Returns a list of maps (keys: classname and filename)
+    Returns:
+        A list of maps (keys: classname and filename).
     """
     import xml.etree.ElementTree as ET
     tree = ET.parse(tagfile)
@@ -469,14 +498,19 @@ def write_mapping_to_php(mapping, outputfile, varname='map'):
     """Write a mapping out as PHP code
 
     Creates a PHP array as described by mapping.  For example, the mapping
-      [("foo","bar"),("x","y")]
+
+        [("foo","bar"),("x","y")]
+
     would cause the file
-      <?php $map = array('foo' => 'bar','x' => 'y') ?>
+
+        <?php $map = array('foo' => 'bar','x' => 'y') ?>
+
     to be written out.
 
-    mapping    -- a list of pairs of strings
-    outputfile -- the file to write to
-    varname    -- override the PHP variable name (defaults to 'map')
+    Args:
+        mapping:    a list of pairs of strings
+        outputfile: the file to write to
+        varname:    override the PHP variable name (defaults to 'map')
     """
     logging.info('Generating PHP mapping')
     with codecs.open(outputfile, 'w', 'utf-8') as f:
@@ -511,7 +545,7 @@ def generate_apidocs(ctx, tmp_dir, doxyfile_entries=None, keep_temp_dirs=False):
     def find_src_subdir(dirlist, deeper_subd=None):
         returnlist = []
         for d in dirlist:
-            pth = os.path.join(ctx.fwinfo['path'], d)
+            pth = os.path.join(ctx.fwinfo.path, d)
             if deeper_subd is not None:
                 pth = os.path.join(pth, deeper_subd)
             if os.path.isdir(pth) or os.path.isfile(pth):
@@ -520,20 +554,14 @@ def generate_apidocs(ctx, tmp_dir, doxyfile_entries=None, keep_temp_dirs=False):
                 pass  # We drop it
         return returnlist
 
-    # Paths and basic project info
-
-    # FIXME: preprocessing?
-    # What about providing the build directory? We could get the version
-    # as well, then.
-
     input_list = []
-    if os.path.isfile(ctx.fwinfo['path']+"/README.md"):
-        input_list.append(ctx.fwinfo['path']+"/README.md")
-    if os.path.isfile(ctx.fwinfo['path']+"/Mainpage.dox"):
-        input_list.append(ctx.fwinfo['path']+"/Mainpage.dox")
+    if os.path.isfile(ctx.fwinfo.path + "/Mainpage.dox"):
+        input_list.append(ctx.fwinfo.path + "/Mainpage.dox")
+    elif os.path.isfile(ctx.fwinfo.path + "/README.md"):
+        input_list.append(ctx.fwinfo.path + "/README.md")
 
-    input_list.extend(find_src_subdir(ctx.fwinfo['srcdirs']))
-    input_list.extend(find_src_subdir(ctx.fwinfo['docdir']))
+    input_list.extend(find_src_subdir(ctx.fwinfo.srcdirs))
+    input_list.extend(find_src_subdir(ctx.fwinfo.docdir))
     image_path_list = []
 
     if ctx.dependency_diagram:
@@ -556,11 +584,11 @@ def generate_apidocs(ctx, tmp_dir, doxyfile_entries=None, keep_temp_dirs=False):
         # FIXME: can we get the project version from CMake? No from GIT TAGS!
 
         # Input locations
-        image_path_list.extend(find_src_subdir(ctx.fwinfo['docdir'], 'pics'))
+        image_path_list.extend(find_src_subdir(ctx.fwinfo.docdir, 'pics'))
         writer.write_entries(
                 INPUT=input_list,
-                DOTFILE_DIRS=find_src_subdir(ctx.fwinfo['docdir'], 'dot'),
-                EXAMPLE_PATH=find_src_subdir(ctx.fwinfo['exampledir']),
+                DOTFILE_DIRS=find_src_subdir(ctx.fwinfo.docdir, 'dot'),
+                EXAMPLE_PATH=find_src_subdir(ctx.fwinfo.exampledir),
                 IMAGE_PATH=image_path_list)
 
         # Other input settings
@@ -579,6 +607,11 @@ def generate_apidocs(ctx, tmp_dir, doxyfile_entries=None, keep_temp_dirs=False):
                 HTML_FOOTER=ctx.doxdatadir + '/footer.html'
                 )
 
+        # Set a layout so that properties are first
+        writer.write_entries(
+            LAYOUT_FILE=ctx.doxdatadir + '/DoxygenLayout.xml'
+            )
+
         # Always write these, even if QHP is disabled, in case Doxygen.local
         # overrides it
         writer.write_entries(
@@ -588,15 +621,16 @@ def generate_apidocs(ctx, tmp_dir, doxyfile_entries=None, keep_temp_dirs=False):
 
         writer.write_entries(
                 GENERATE_MAN=ctx.man_pages,
-                GENERATE_QHP=ctx.qhp,
-                SEARCHENGINE=ctx.searchengine)
+                GENERATE_QHP=ctx.qhp)
+
+                #, SEARCHENGINE=ctx.searchengine)
 
         if doxyfile_entries:
             writer.write_entries(**doxyfile_entries)
 
         # Module-specific overrides
-        if find_src_subdir(ctx.fwinfo['docdir']):
-            localdoxyfile = os.path.join(find_src_subdir(ctx.fwinfo['docdir'])[0], 'Doxyfile.local')
+        if find_src_subdir(ctx.fwinfo.docdir):
+            localdoxyfile = os.path.join(find_src_subdir(ctx.fwinfo.docdir)[0], 'Doxyfile.local')
             if os.path.isfile(localdoxyfile):
                 with codecs.open(localdoxyfile, 'r', 'utf-8') as f:
                     for line in f:
@@ -607,14 +641,12 @@ def generate_apidocs(ctx, tmp_dir, doxyfile_entries=None, keep_temp_dirs=False):
 
 
 def postprocess(ctx, classmap, template_mapping=None):
-    # TODO: copyright must be set from outside
-    copyright = '1996-' + str(datetime.date.today().year) + ' The KDE developers'
     mapping = {
             'doxygencss': 'doxygen.css',
             'resources': ctx.resourcedir,
             'title': ctx.title,
             'fwinfo': ctx.fwinfo,
-            'copyright': copyright,
+            'copyright': ctx.copyright,
             'api_searchbox': ctx.api_searchbox,
             'doxygen_menu': {'entries': menu_items(ctx.htmldir, ctx.modulename)},
             'class_map': {'classes': classmap},
@@ -671,21 +703,33 @@ def generate_diagram(png_path, fancyname, dot_files, tmp_dir):
     return True
 
 
-def create_fw_context(args, lib, tagfiles):
+def create_fw_context(args, lib, tagfiles, copyright=''):
+
+    # There is one more level for groups
+    if lib.part_of_group:
+        corrected_tagfiles = []
+        for k in range(len(tagfiles)):
+            if tagfiles[k][1].startswith("http://") or tagfiles[k][1].startswith("https://"):
+                corrected_tagfiles.append(tagfiles[k])
+            else:
+                corrected_tagfiles.append((tagfiles[k][0], '../' + tagfiles[k][1]))
+    else:
+        corrected_tagfiles = tagfiles
+
     return Context(args,
                    # Names
-                   modulename=lib['name'],
-                   fancyname=lib['fancyname'],
+                   modulename=lib.name,
+                   fancyname=lib.fancyname,
                    fwinfo=lib,
                    # KApidox files
-                   resourcedir=('../../resources' if lib['parent'] is None
-                                else '../../../resources'),
+                   resourcedir=('../../../resources' if lib.part_of_group
+                                else '../../resources'),
                    # Input
-                   #srcdir=lib['srcdir'],
-                   tagfiles=tagfiles,
-                   dependency_diagram=lib['dependency_diagram'],
+                   copyright=copyright,
+                   tagfiles=corrected_tagfiles,
+                   dependency_diagram=lib.dependency_diagram,
                    # Output
-                   outputdir=lib['outputdir'],
+                   outputdir=lib.outputdir,
                    )
 
 
@@ -697,20 +741,32 @@ def gen_fw_apidocs(ctx, tmp_base_dir):
                      doxyfile_entries=dict(WARN_IF_UNDOCUMENTED=True)
                      )
 
+def create_fw_tagfile_tuple(lib):
+    tagfile = os.path.abspath(
+                os.path.join(
+                    lib.outputdir,
+                    'html',
+                    lib.fancyname+'.tags'))
+    if lib.part_of_group:
+        prefix = '../../'
+    else:
+        prefix = '../../'
+    return (tagfile, prefix + lib.outputdir + '/html/')
+
 
 def finish_fw_apidocs(ctx, group_menu):
     classmap = build_classmap(ctx.tagfile)
-    write_mapping_to_php(classmap, os.path.join(ctx.outputdir, 'classmap.inc'))
+    #write_mapping_to_php(classmap, os.path.join(ctx.outputdir, 'classmap.inc'))
 
     entries = [{
         'href': '../../index.html',
         'text': 'KDE API Reference'
         }]
-    if ctx.fwinfo['parent'] is not None:
+    if  ctx.fwinfo.part_of_group:
         entries[0]['href'] = '../' + entries[0]['href']
         entries.append({
             'href': '../../index.html',
-            'text': ctx.fwinfo['product']['fancyname']
+            'text': ctx.fwinfo.product.fancyname
             })
     entries.append({
         'href': 'index.html',
@@ -738,14 +794,171 @@ def finish_fw_apidocs(ctx, group_menu):
         mapping.update(template_mapping)
     logging.info('Postprocessing')
 
-    tmpl = create_jinja_environment(ctx.doxdatadir).get_template('doxygen.html')
+    tmpl = create_jinja_environment(ctx.doxdatadir).get_template('library.html')
     postprocess_internal(ctx.htmldir, tmpl, mapping)
 
+    tmpl2 = create_jinja_environment(ctx.doxdatadir).get_template('search.html')
+    search_output = ctx.fwinfo.outputdir + "/html/search.html"
+    with codecs.open(search_output, 'w', 'utf-8') as outf:
+        outf.write(tmpl2.render(mapping))
 
-def create_fw_tagfile_tuple(lib):
-    tagfile = os.path.abspath(
-                os.path.join(
-                    lib['outputdir'],
-                    'html',
-                    lib['fancyname']+'.tags'))
-    return (tagfile, '../../' + lib['outputdir'] + '/html/')
+
+def indexer(lib):
+    """ Create json index from xml
+      <add>
+        <doc>
+          <field name="type">source</field>
+          <field name="name">kcmodule.cpp</field>
+          <field name="url">kcmodule_8cpp_source.html#l00001</field>
+          <field name="keywords"></field>
+          <field name="text"></field>
+        </doc>
+      </add
+    """
+
+    doclist = []
+    tree = xmlET.parse(lib.outputdir + '/searchdata.xml')
+    for doc_child in tree.getroot():
+        field = {}
+        for child in doc_child:
+            if child.attrib['name'] == "type":
+                if child.text == 'source':
+                    field = None
+                    break; # We go to next <doc>
+                field['type'] = child.text
+            elif child.attrib['name'] == "name":
+                field['name'] = child.text
+            elif child.attrib['name'] == "url":
+                field['url'] = child.text
+            elif child.attrib['name'] == "keywords":
+                field['keyword'] = child.text
+            elif child.attrib['name'] == "text":
+                field['text'] = "" if child.text is None else child.text
+        if field is not None:
+            doclist.append(field)
+
+    indexdic = {
+        'name': lib.name,
+        'fancyname': lib.fancyname,
+        'docfields': doclist
+        }
+
+    with open(lib.outputdir + '/html/searchdata.json', 'w') as f:
+        for chunk in json.JSONEncoder().iterencode(indexdic):
+            f.write(chunk)
+
+def create_product_index(product):
+    doclist = []
+    for lib in product.libraries:
+        with open(lib.outputdir+'/html/searchdata.json', 'r') as f:
+            libindex = json.load(f)
+            for item in libindex['docfields']:
+                if lib.part_of_group:
+                    item['url'] = lib.name + '/html/' + item['url']
+                else:
+                    item['url'] = 'html/' + item['url']
+            doclist.append(libindex)
+
+    indexdic = {
+        'name': product.name,
+        'fancyname': product.fancyname,
+        'libraries': doclist
+        }
+
+    with open(product.outputdir + '/searchdata.json', 'w') as f:
+        for chunk in json.JSONEncoder().iterencode(indexdic):
+            f.write(chunk)
+
+def create_global_index(products):
+    doclist = []
+    for product in products:
+        with open(product.outputdir+'/searchdata.json', 'r') as f:
+            prodindex = json.load(f)
+            for proditem in prodindex['libraries']:
+                for item in proditem['docfields']:
+                    item['url'] = product.name + '/' + item['url']
+            doclist.append(prodindex)
+
+    indexdic = {
+        'all': doclist
+        }
+    with open('searchdata.json', 'w') as f:
+        for chunk in json.JSONEncoder().iterencode(indexdic):
+            f.write(chunk)
+
+def create_qch(products, tagfiles):
+    tag_root = "QtHelpProject"
+    tag_files = "files"
+    tag_filterSection = "filterSection"
+    tag_keywords = "keywords"
+    tag_toc = "toc"
+    for product in products:
+        tree_out = ET.ElementTree(ET.Element("QtHelpProject"))
+        root_out = tree_out.getroot()
+        root_out.set("version", "1.0")
+        namespace = ET.SubElement(root_out, "namespace")
+        namespace.text = "org.kde." + product.name
+        virtualFolder = ET.SubElement(root_out, "virtualFolder")
+        virtualFolder.text = product.name
+        filterSection = ET.SubElement(root_out, tag_filterSection)
+        filterAttribute = ET.SubElement(filterSection, "filterAttribute")
+        filterAttribute.text = "doxygen"
+        toc = ET.SubElement(filterSection, "toc")
+        keywords = ET.SubElement(filterSection, tag_keywords)
+        if len(product.libraries) > 0:
+            if product.libraries[0].part_of_group:
+                product_indexSection = ET.SubElement(toc, "section", {'ref': product.name + "/index.html", 'title': product.fancyname})
+        files = ET.SubElement(filterSection, tag_files)
+
+        for lib in sorted(product.libraries, key=lambda lib: lib.name):
+            tree = ET.parse(lib.outputdir + '/html/index.qhp')
+            root = tree.getroot()
+            for child in root.findall(".//*[@ref]"):
+                if lib.part_of_group:
+                    child.attrib['ref'] = lib.name + "/html/" + child.attrib['ref']
+                else:
+                    child.attrib['ref'] = "html/" + child.attrib['ref']
+                child.attrib['ref'] = product.name + '/' +child.attrib['ref']
+
+            for child in root.find(".//"+tag_toc):
+                print(child.attrib['ref'])
+                if lib.part_of_group:
+                    product_indexSection.append(child)
+                else:
+                    toc.append(child)
+
+            for child in root.find(".//keywords"):
+                keywords.append(child)
+
+            resources = [
+                "*.json",
+                product.name + "/*.json",
+                product.name +"/" + lib.name +"/html/*.json",
+                product.name + "/*.html",
+                product.name + "/html/*.html",
+                product.name +"/html/*.png",
+                product.name +"/html/*.css",
+                product.name +"/html/*.js",
+                product.name +"/" + lib.name +"/html/*.html",
+                product.name +"/" + lib.name +"/html/*.png",
+                product.name +"/" + lib.name +"/html/*.css",
+                product.name +"/" + lib.name +"/html/*.js",
+                "resources/css/*.css",
+                "resources/3rd-party/bootstrap/css/*.css",
+                "resources/3rd-party/jquery/jquery-3.1.0.min.js",
+                "resources/*.svg",
+                "resources/js/*.js",
+                "resources/icons/*",
+            ]
+            for resource in resources:
+                file_elem = ET.SubElement(files, "file")
+                file_elem.text = resource
+
+        if not os.path.isdir('qch'):
+            os.mkdir('qch')
+
+        name = product.name+".qhp"
+        outname = product.name+".qch"
+        tree_out.write(name, encoding="utf-8", xml_declaration=True)
+        subprocess.call(["qhelpgenerator", name, '-o', 'qch/'+outname])
+        os.remove(name)
